import PDFDoc from 'pdfkit';
import { PDFTools } from '../helper/index.js';
import fs from 'fs';
import os from 'os';
import path from 'path';
class PDF {
    constructor(doc, title, outputPath) {
        this.doc = doc;
        this.title = title;
        this.outputPath = outputPath;
    }
    static create(options, outputPath, autoPipe = true, onFinish = () => { }) {
        const defaultOptions = PDFTools.definitions.page;
        const docOptions = {
            ...defaultOptions,
            ...options,
        };
        const title = docOptions.info?.Title || 'Relatório';
        const document = new PDFDoc(docOptions);
        if (autoPipe) {
            const tempDir = os.tmpdir();
            const filePath = outputPath || path.join(tempDir, `report-${Date.now()}.pdf`);
            const stream = document.pipe(fs.createWriteStream(filePath));
            stream.on('finish', onFinish);
            PDFTools.addHeaderPage(document, title, 1);
        }
        return new PDF(document, title, outputPath);
    }
    getDocument() {
        return this.doc;
    }
    splitCustomStyledText(str, tagToFind = '**') {
        let tagIdx = str.indexOf(tagToFind);
        if (tagIdx === -1)
            return str;
        const strParts = [];
        let strToVerify = str;
        const customStyledText = [];
        const tagSize = tagToFind.length;
        const lastClosedTagIdx = str.lastIndexOf(tagToFind);
        while (tagIdx !== -1) {
            strParts.push(strToVerify.substring(0, tagIdx));
            const closeTagIdx = strToVerify.indexOf(tagToFind, tagIdx + 1);
            customStyledText.push(strToVerify.substring(tagIdx + tagSize, closeTagIdx));
            strToVerify = strToVerify.substring(closeTagIdx + tagSize);
            tagIdx = strToVerify.indexOf(tagToFind);
        }
        if (lastClosedTagIdx + tagSize !== str.length) {
            strParts.push(str.substring(lastClosedTagIdx + tagSize));
        }
        return { strParts, customStyledText };
    }
    applyBoldStyle(text, options) {
        const result = this.splitCustomStyledText(text);
        const { x, y, height, width } = options;
        if (typeof result === 'string') {
            this.doc.text(text, x, y, options);
            return;
        }
        const { strParts, customStyledText } = result;
        const startText = strParts.shift();
        if (typeof startText !== 'string')
            return;
        this.doc.text(startText, x, y, { continued: true, height, width });
        strParts.forEach((str) => {
            const styledText = customStyledText.shift();
            if (styledText) {
                console.log(`**${styledText}**`);
                this.doc.font('Helvetica-Bold').text(styledText);
            }
            this.doc.font('Helvetica').text(str);
        });
        this.doc.font('Helvetica-Bold');
        while (customStyledText.length) {
            const styledText = customStyledText.shift();
            if (styledText) {
                console.log(`-${styledText}-`);
                this.doc.text(styledText, {});
            }
        }
        this.doc.font('Helvetica');
    }
    applyCustomStyledText(text, options) {
        this.applyBoldStyle(text, options);
    }
    drawTableCell(text, options) {
        const { x, y, height, width, color, align, bold = false } = options;
        if (color) {
            this.doc.rect(x, y, width, height).fill(color);
        }
        this.doc.fontSize(12).fillColor('black');
        if (bold) {
            this.doc.font('Helvetica-Bold');
        }
        else {
            this.doc.font('Helvetica');
        }
        this.doc.text(text, x + 5, y + 5, {
            width: width - 10,
            height: height - 10,
            align: align || 'left',
        });
    }
    drawTableHeader(options) {
        const { x, y, height, width, label, align = 'left' } = options;
        const color = '#D3D3D3';
        this.drawTableCell(label, {
            x,
            y,
            width,
            height,
            color,
            align,
            bold: true,
        });
    }
    calculateColumnWidth(header, data, maxWidth) {
        const columnsWidth = [];
        const getWidthByTextSize = (text) => this.doc.widthOfString(text);
        header.forEach((text) => {
            const width = getWidthByTextSize(text);
            columnsWidth.push(width);
        });
        data.forEach((rowData) => {
            rowData.forEach((cellValue, columnIdx) => {
                const tempWidth = getWidthByTextSize(cellValue);
                if (tempWidth > columnsWidth[columnIdx]) {
                    columnsWidth[columnIdx] = tempWidth;
                }
            });
        });
        const fullSize = columnsWidth.reduce((fullWidth, size) => fullWidth + size, 0);
        return columnsWidth.map((size) => Math.trunc((size / fullSize) * maxWidth));
    }
    addTable(header, data, options) {
        const { x, y } = options;
        const height = 30;
        let currentY = y;
        let lineCount = 1;
        const currentPage = this.doc.page;
        const availableWidth = currentPage.width - currentPage.margins.left - currentPage.margins.right;
        let pageCount = 1;
        const columnsWidth = this.calculateColumnWidth(header, data, availableWidth);
        const tableWidth = columnsWidth.reduce((fullSize, columnWidth) => fullSize + columnWidth, 0);
        const addTableHeader = ({ startX = x, startY = y } = {}) => {
            let headerX = startX;
            header.forEach((label, index) => {
                const align = index === 0 ? 'left' : 'right';
                this.drawTableHeader({
                    x: headerX,
                    y: startY,
                    width: columnsWidth[index],
                    height,
                    label,
                    align,
                });
                headerX += columnsWidth[index];
            });
        };
        addTableHeader();
        data.forEach((rowData) => {
            let rowY = currentY + lineCount * height;
            if (rowY > this.doc.page.height - 50) {
                this.doc.addPage();
                pageCount += 1;
                PDFTools.addHeaderPage(this.doc, this.title, pageCount);
                lineCount = 1;
                currentY = y;
                addTableHeader();
                rowY = currentY + lineCount * height;
            }
            lineCount += 1;
            this.doc
                .moveTo(x, rowY)
                .lineTo(x + tableWidth, rowY)
                .stroke();
            let dataPositionX = x;
            rowData.forEach((row, index) => {
                const align = index === 0 ? 'left' : 'right';
                const cellWidth = columnsWidth[index];
                this.drawTableCell(row, {
                    x: dataPositionX,
                    y: rowY,
                    width: cellWidth,
                    height,
                    align,
                });
                dataPositionX += cellWidth;
            });
        });
    }
}
export { PDF };
export default PDF;
