import { Request, Sftp } from '../util/index.js';
import { format } from 'date-fns';
import prettyBytes from 'pretty-bytes';
import SemVer from 'semver';
class AssistantInstaller {
    constructor() {
        this.sftpParams = {};
        this.prodVersion = 'AssistantInstaller.exe';
        this.prodVersionInfo = 'installer-version.json';
        this.bkpProdVersion = 'AssistantInstaller.bkp.exe';
        this.bkpProdVersionInfo = 'installer-version.bkp.json';
        const { SFTP_HOST_INSTALLER_PKG: host, SFTP_PORT_INSTALLER_PKG: port, SFTP_USER_INSTALLER_PKG: user, SFTP_PASSWORD_INSTALLER_PKG: password, } = process.env;
        this.sftpParams = {
            host,
            port: port ? Number(port) : 22,
            user,
            password,
        };
    }
    async getApprovedVersion(clientType) {
        try {
            const { ASSIST_INSTALLER_PACKAGE_URL: pkgServer, SFTP_PATH_SAAS_INSTALLER_PKG: saasPath, SFTP_PATH_EXT_INSTALLER_PKG: externalPath, } = process.env;
            if (!pkgServer)
                throw new Error('Invalid {ASSIST_INSTALLER_PACKAGE_URL}');
            const versionPath = clientType.toUpperCase() === 'SAAS' ? saasPath : externalPath;
            const url = Request.removeStartEndBar(pkgServer);
            const urlPath = Request.removeStartEndBar(versionPath || '');
            const { version, published } = await Request.get(`${url}/${urlPath}/installer-version.json`);
            return { version, published };
        }
        catch (err) {
            return { version: '0.0.0', error: err };
        }
    }
    async createRemotePaths(client) {
        const { CREATED_PATH_INSTALLER_PKG: createdPath, SFTP_PATH_SAAS_INSTALLER_PKG: saasPath, SFTP_PATH_EXT_INSTALLER_PKG: externalPath, } = process.env;
        if (createdPath)
            return;
        if (saasPath) {
            await client.mkdir(saasPath, true);
        }
        if (externalPath) {
            await client.mkdir(externalPath, true);
        }
        process.env.CREATED_PATH_INSTALLER_PKG = 'YES';
    }
    async getVersionInfo() {
        try {
            const { ASSIST_INSTALLER_PACKAGE_URL: pkgServer, SFTP_PATH_SAAS_INSTALLER_PKG: saasPath, SFTP_PATH_EXT_INSTALLER_PKG: externalPath, } = process.env;
            if (!pkgServer)
                throw new Error('Invalid {ASSIST_INSTALLER_PACKAGE_URL}');
            const url = Request.removeStartEndBar(pkgServer);
            const defaultErrorHandler = (err) => ({ version: '0.0.0', err });
            const saasInfo = await Request.get(`${url}/${saasPath}/installer-version.json`).catch(defaultErrorHandler);
            const saasBkpInfo = await Request.get(`${url}/${saasPath}/installer-version.bkp.json`).catch(defaultErrorHandler);
            const externalInfo = await Request.get(`${url}/${externalPath}/installer-version.json`).catch(defaultErrorHandler);
            const externalBkpInfo = await Request.get(`${url}/${externalPath}/installer-version.bkp.json`).catch(defaultErrorHandler);
            return {
                saas: {
                    production: saasInfo,
                    productionBkp: saasBkpInfo,
                },
                external: {
                    production: externalInfo,
                    productionBkp: externalBkpInfo,
                },
            };
        }
        catch (err) {
            return { version: '0.0.0', error: err };
        }
    }
    async list() {
        const { client } = await Sftp.connect(this.sftpParams);
        await this.createRemotePaths(client);
        const { SFTP_PATH_SAAS_INSTALLER_PKG: saasPath, SFTP_PATH_EXT_INSTALLER_PKG: externalPath, } = process.env;
        const prodVersionInfo = await this.getVersionInfo();
        let saasVersions = [];
        let externalVersions = [];
        if (saasPath) {
            saasVersions = await client.list(saasPath);
        }
        if (externalPath) {
            externalVersions = await client.list(externalPath);
        }
        const versions = [];
        const handleVersionInfo = (name, clientType) => {
            if (prodVersionInfo.error)
                return '';
            const { saas, external } = prodVersionInfo;
            if (name === this.prodVersion) {
                if (clientType === 'saas') {
                    return saas?.production?.version !== '0.0.0' ? saas?.production?.version : '';
                }
                return external?.production?.version !== '0.0.0' ? external?.production?.version : '';
            }
            if (name === this.bkpProdVersion) {
                if (clientType === 'saas') {
                    return saas?.productionBkp?.version !== '0.0.0' ? saas?.productionBkp?.version : '';
                }
                return external?.productionBkp?.version !== '0.0.0' ? external?.productionBkp?.version : '';
            }
        };
        const handleVersion = (file, versionPath, clientType) => {
            if (file.name.endsWith('.exe')) {
                return ({
                    path: versionPath || '',
                    size: prettyBytes(file.size),
                    name: file.name,
                    modifyTime: format(new Date(file.modifyTime), 'dd/MM/yyyy HH:mm'),
                    clientType,
                    version: handleVersionInfo(file.name, clientType),
                });
            }
            return null;
        };
        saasVersions.forEach((file) => {
            const version = handleVersion(file, saasPath || '', 'saas');
            if (version)
                versions.push(version);
        });
        externalVersions.forEach((file) => {
            const version = handleVersion(file, externalPath || '', 'external');
            if (version)
                versions.push(version);
        });
        return versions;
    }
    async createBkpVersion(client, versionPath) {
        const currentVersion = await client.exists(`${versionPath}/${this.prodVersion}`);
        if (currentVersion === '-') {
            await client.delete(`${versionPath}/${this.bkpProdVersion}`, true);
            await client.delete(`${versionPath}/${this.bkpProdVersionInfo}`, true);
            await client.rename(`${versionPath}/${this.prodVersion}`, `${versionPath}/${this.bkpProdVersion}`);
            const currentVersionInfo = await client.exists(`${versionPath}/${this.prodVersionInfo}`);
            if (currentVersionInfo === '-') {
                await client.rename(`${versionPath}/${this.prodVersionInfo}`, `${versionPath}/${this.bkpProdVersionInfo}`);
            }
        }
    }
    async create(file, params) {
        const { client } = await Sftp.connect(this.sftpParams);
        const versionPath = ({
            saas: process.env.SFTP_PATH_SAAS_INSTALLER_PKG,
            external: process.env.SFTP_PATH_EXT_INSTALLER_PKG,
        })[params.client];
        if (!versionPath)
            throw new Error('INVALID_PATH_TO_SAVE_VERSION');
        const writeStreamOptions = {
            encoding: null,
            mode: 0o755,
        };
        await this.createBkpVersion(client, versionPath);
        await client.put(file.buffer, `${versionPath}/${this.prodVersion}`, { writeStreamOptions });
        const versionInfo = JSON.stringify({
            version: params.version,
            published: format(new Date(), 'dd/MM/yyyy HH:mm'),
        }, null, 2);
        await client.put(Buffer.from(versionInfo, 'utf8'), `${versionPath}/${this.prodVersionInfo}`, { writeStreamOptions });
        return {
            status: 'SUCCESS_UPLOAD_VERSION',
        };
    }
    getVersionDetail(clientType) {
        const versionPath = ({
            saas: process.env.SFTP_PATH_SAAS_INSTALLER_PKG,
            external: process.env.SFTP_PATH_EXT_INSTALLER_PKG,
        })[clientType];
        return {
            versionPath,
            prodVersion: this.prodVersion,
            prodVersionInfo: this.prodVersionInfo,
            bkpProdVersion: this.bkpProdVersion,
            bkpProdVersionInfo: this.bkpProdVersionInfo,
        };
    }
    async remove(params) {
        if (!Array.isArray(params.versionsToRemove))
            throw new Error('INVALID_VERSIONS_TO_REMOVE');
        const { client } = await Sftp.connect(this.sftpParams);
        const deletePromiseArr = [];
        params.versionsToRemove.forEach(async (version) => {
            const versionPath = ({
                saas: process.env.SFTP_PATH_SAAS_INSTALLER_PKG,
                external: process.env.SFTP_PATH_EXT_INSTALLER_PKG,
            })[version.clientType];
            deletePromiseArr.push(client.delete(`${versionPath}/${version.name}`, true).then(() => {
                const versionInfo = version.name === this.prodVersion ? this.prodVersionInfo : this.bkpProdVersionInfo;
                return client.delete(`${versionPath}/${versionInfo}`, true);
            }));
        });
        await Promise.all(deletePromiseArr);
    }
    async restoreBackup(params) {
        if (!params.versionToRestore)
            throw new Error('INVALID_VERSION_TO_RESTORE');
        const { clientType } = params.versionToRestore;
        const { prodVersion, prodVersionInfo, versionPath, bkpProdVersion, bkpProdVersionInfo } = this.getVersionDetail(clientType);
        const { client } = await Sftp.connect(this.sftpParams);
        await client.delete(`${versionPath}/${prodVersion}`, true);
        await client.delete(`${versionPath}/${prodVersionInfo}`, true);
        await client.rename(`${versionPath}/${bkpProdVersion}`, `${versionPath}/${prodVersion}`);
        await client.rename(`${versionPath}/${bkpProdVersionInfo}`, `${versionPath}/${prodVersionInfo}`);
    }
    validateAgentVersion(version) {
        const { MIN_INSTALLER_VERSION = '' } = process.env;
        const isValid = version && MIN_INSTALLER_VERSION && SemVer.gte(version, MIN_INSTALLER_VERSION);
        if (!isValid) {
            throw new Error('INVALID_INSTALLER_VERSION');
        }
    }
}
const instance = new AssistantInstaller();
export { instance as AssistantInstaller };
export default instance;
