import path from 'path';
import { PrismaClient } from '../db/index.js';
import { File, Dependency } from '../util/index.js';
import { License } from '../integration/index.js';
class Product {
    async create(params) {
        const { devopsId, productName, requiredProducts, description = '', productImage = '', productOriginalImage = '', requirements = '', guideUrl = '', } = params;
        const data = {
            devopsId,
            productName,
            description,
            productImage,
            productOriginalImage,
            requirements,
            guideUrl,
            requiredProducts: {
                connect: requiredProducts?.map((productId) => ({ id: productId })),
            },
        };
        return PrismaClient.product.create({ data, include: { requiredProducts: true } });
    }
    async list() {
        return PrismaClient.product.findMany({
            include: { requiredProducts: true },
        });
    }
    async listByDevopsId(devopsIdCollection) {
        if (!devopsIdCollection)
            throw new Error('Invalid devopsIdCollection');
        return PrismaClient.product.findMany({
            where: { devopsId: { in: devopsIdCollection } },
            include: { requiredProducts: true },
        });
    }
    async removeProductImage(fileName, originalImage) {
        if (!process.env.IMAGES_PATH) {
            throw new Error('Invalid IMAGES_PATH');
        }
        const imgPath = process.env.IMAGES_PATH;
        const resizedImgPath = path.join(process.env.IMAGES_PATH, 'resized');
        await new File(fileName, resizedImgPath).remove();
        if (originalImage) {
            await new File(originalImage, imgPath).remove();
        }
    }
    async remove({ productsToRemove }) {
        const products = Array.isArray(productsToRemove) ? productsToRemove : [];
        const where = { devopsId: { in: products.map((product) => product.devopsId) } };
        const result = await PrismaClient.product.findMany({
            where: {
                requiredProducts: {
                    some: {
                        id: {
                            in: productsToRemove?.map((product) => product.id),
                        },
                    },
                },
            },
        });
        if (result && result.length) {
            return {
                error: 'PRODUCT_HAS_DEPENDENTS',
                data: result,
            };
        }
        const response = PrismaClient.product.deleteMany({ where });
        const promiseRemoveImages = [];
        productsToRemove?.forEach((product) => {
            if (product.productImage) {
                promiseRemoveImages.push(this.removeProductImage(product.productImage, product.productOriginalImage));
            }
        });
        await Promise.allSettled(promiseRemoveImages).then((imgDeleteResult) => {
            imgDeleteResult.forEach((deleteResult) => {
                if (deleteResult.status === 'rejected') {
                    console.error(deleteResult.reason);
                }
            });
        });
        return response;
    }
    async hasCircularDependency(updatedProduct) {
        const { devopsId, requiredProducts } = updatedProduct;
        if (!Array.isArray(requiredProducts) || !requiredProducts.length) {
            return { result: false, circularDependency: [] };
        }
        const allProducts = await this.list();
        const dependencyList = allProducts.reduce((dependency, product) => {
            if (product.devopsId.toString() === devopsId.toString()) {
                dependency[product.id] = requiredProducts;
            }
            else {
                dependency[product.id] = product.requiredProducts.map((requiredProduct) => requiredProduct.id);
            }
            return dependency;
        }, {});
        const circularDependency = Dependency.findCircularDependencies(dependencyList);
        if (circularDependency && circularDependency.length) {
            const [firstError] = circularDependency;
            return {
                result: true,
                circularDependency: allProducts.filter((product) => firstError.includes(product.id)),
            };
        }
        return {
            result: false,
            circularDependency: [],
        };
    }
    async modify(params) {
        const { devopsId, requiredProducts: productsId, ...otherParams } = params;
        let requiredProducts;
        if (Array.isArray(productsId)) {
            const onlyIds = productsId.every((item) => typeof item === 'string');
            requiredProducts = {
                set: onlyIds ? productsId.map((id) => ({ id })) : productsId.map((product) => ({ id: product.id })),
            };
        }
        const productDependencyResult = await this.hasCircularDependency({
            devopsId,
            requiredProducts: productsId,
        });
        if (productDependencyResult.result) {
            const { circularDependency } = productDependencyResult;
            return { circularDependency };
        }
        await PrismaClient.product.update({
            where: { devopsId: Number(devopsId) },
            data: { ...otherParams, requiredProducts },
        });
        const savedRow = await PrismaClient.product.findUnique({
            where: { devopsId },
            include: { requiredProducts: true },
        });
        return savedRow;
    }
    async licenseSync() {
        const [savedProducts, licenseProducts] = await Promise.all([
            PrismaClient.product.findMany(),
            License.getProductsByOrg({ NRORG: '20' }),
        ]);
        const products = savedProducts.reduce((result, product) => {
            result[product.devopsId] = product.productName;
            return result;
        }, {});
        const result = await Promise.allSettled(licenseProducts.map(async ({ ID, NMPRODUTO }) => {
            if ((ID.toString() in products) && products[ID] !== NMPRODUTO) {
                return PrismaClient.product.update({
                    data: {
                        productName: NMPRODUTO,
                    },
                    where: { devopsId: ID },
                });
            }
            console.log(ID, products);
            if (!(ID.toString() in products)) {
                console.log(`${ID} - ${NMPRODUTO} | ${(ID in products)}`, products);
                return PrismaClient.product.create({
                    data: {
                        devopsId: parseInt(ID, 10),
                        productName: NMPRODUTO,
                    },
                });
            }
        }));
        const errors = result.filter((dbResult) => dbResult.status === 'rejected').map((error) => error.reason);
        if (errors.length) {
            console.log(errors);
            throw new Error('UNEXPECTED_ERROR_PRODUCT_SYNC');
        }
    }
}
const instance = new Product();
export { instance as Product };
export default instance;
